#pragma once

#include <fmt/format.h>
#include <glib.h>

#define LICENSE_NEW_INTERFACE
#include "license_i.h"

#include <functional>

namespace Kiran
{
#define MAX_LINE 50

#define IP4_ADDR_LEN 20
#define LICENSE_TCP_PORT 7788
// 和服务器通讯的超时设置
#define LICENSE_SOCKET_TIMEOUT 5
#define LICENSE_SOCKET_DATA_MAX_LEN 10240

/* -------------------------- license -------------------- */
// 密钥长度
#define KEY_LEN 16
#define DATE_LENGTH 3
// 机器码长度
#define MACHINE_CODE_LENGTH 8
// 过期日期长度
#define EXPIRED_DATE_LENGTH 3
// 客户代码长度
#define CUSTOMER_CODE_LENGTH 5
// 注册日期长度
#define REGISTER_DATE_LENGTH 3
// 注册类型长度
#define REGISTER_TYPE_LENGTH 1

// 激活码的字节长度
#define LICENSE_CODE_LENGTH \
    (MACHINE_CODE_LENGTH + EXPIRED_DATE_LENGTH + CUSTOMER_CODE_LENGTH)

// 激活码位长度
#define LICENSE_CODE_BIT_LENGTH (LICENSE_CODE_LENGTH * 8)

// 激活码显示长度
#define LICENSE_CODE_SHOW_LENGHT 25
// 以前的显示长度时32位
#define LICENSE_CODE_LONG_SHOW_LENGHT 32
// 激活码显示进制数
#define LICENSE_CODE_SHOW_BASE 36
// 以前激活码显示进制数
#define LICENSE_CODE_LONG_SHOW_BASE 16

// 10分钟
#define TIMEOUT_IN_SECONDS (600)
// 一天
#define DAY_IN_SECONDS (24 * 3600)
//三个月
#define EXPIRE_IN_DAYS (180)
#define EXPIRE_IN_SECONDS (EXPIRE_IN_DAYS * DAY_IN_SECONDS)

// 操作系统授权名称
#define LICENSE_OBJECT_OS_NAME "KylinSecOS"
// 虚拟UKey名称
#define LICENSE_OBJECT_VUKEY_NAME "VirtualUSBKey"

/* ---------------------- USBKEY -------------------- */
// 开发商ID为8字节
#define VENDOR_ID_BYTES 8

// 开发商口令为24字节
#define DEVELOPER_PIN_BYTES 24

// 最大支持插入的Rockey数量
#define ROCKEY_MAX_NUM 32

// 授权点数
#define RY3_AUTHORIZED_NUM_BYTES 4
// 剩余点数
#define RY3_REMAINING_NUM_BYTES 4
// 客户信息
#define RY3_CUSTOMER_INFO_BYTES 24
// 过期时间
#define RY3_EXPIRED_TIME_BYTES 8
// 硬件序列号偏移(16字节)
#define RY3_HARDINFO_BYTES 16

// 最大可支持授权的机器数
#define UKEY_MAX_AUTHORIZED_NUM 600

#define CLIENT_RY3_VENDOR_ID "92799461"
#define SERVER_RY3_VENDOR_ID "3BA0B50F"

enum UKeyAddressOffset
{
    // 全能版授权点数
    UKEY_ADDRESS_OFFSET_ALMIGHTY_AUTHORIZED_POINTS = 7168,
    // 全能版剩余点数
    UKEY_ADDRESS_OFFSET_ALMIGHTY_REMAINING_POINTS = 7172,
    // 客户信息
    UKEY_ADDRESS_OFFSET_CUSTOMER_INFO = 7176,
    // 过期时间
    UKEY_ADDRESS_OFFSET_EXPIRED_DATE = 7201,
    // 硬件序列号
    UKEY_ADDRESS_OFFSET_HARD_ID = 7209,
    // 服务器激活码（预留字段）
    UKEY_ADDRESS_OFFSET_SERVER_LICENSE_CODE = 7225,
    // 桌面版授权点数
    UKEY_ADDRESS_OFFSET_DESKTOP_AUTHORIZED_POINTS = 7241,
    // 桌面版剩余点数
    UKEY_ADDRESS_OFFSET_DESKTOP_REMAINING_POINTS = 7245,
    // 服务器版授权点数
    UKEY_ADDRESS_OFFSET_SERVER_AUTHORIZED_POINTS = 7249,
    // 服务器版剩余点数
    UKEY_ADDRESS_OFFSET_SERVER_REMAINING_POINTS = 7253,
    // 工控版授权点数
    UKEY_ADDRESS_OFFSET_INDUSTRIAL_AUTHORIZED_POINTS = 7257,
    // 工控版剩余点数
    UKEY_ADDRESS_OFFSET_INDUSTRIAL_REMAINING_POINTS = 7261,
    // 开发版授权点数
    UKEY_ADDRESS_OFFSET_DEVELOPMENT_AUTHORIZED_POINTS = 7265,
    // 开发版剩余点数
    UKEY_ADDRESS_OFFSET_DEVELOPMENT_REMAINING_POINTS = 7269,
    UKEY_ADDRESS_OFFSET_OTHER = 7273,
    UKEY_ADDRESS_OFFSET_LAST = 8192,
};

enum UKeyPresenceType
{
    UKEY_PRESENCE_TYPE_UNKNOWN = 0,
    // 物理UKey设备
    UKEY_PRESENCE_TYPE_PHYSICAL,
    // 虚拟UKey设备
    UKEY_PRESENCE_TYPE_VIRTUAL,
};

enum UKeyPointsIndex
{
    // 全能版
    UKEY_POINTS_INDEX_ALMIGHTY = 0,
    // 桌面版
    UKEY_POINTS_INDEX_DESKTOP,
    // 服务器版
    UKEY_POINTS_INDEX_SERVER,
    // 工控版
    UKEY_POINTS_INDEX_INDUSTRIAL,
    // 开发版
    UKEY_POINTS_INDEX_DEVELOPMENT,
    UKEY_POINTS_INDEX_LAST
};

// 授权点数和剩余点数
struct UKeyPoints
{
    int32_t authorized_points;
    int32_t remaining_points;
};

// ukey 高级存储区数据结构
struct UKeyHighAddress
{
    UKeyHighAddress()
    {
        memset(this->points, 0, sizeof(this->points));
        g_date_clear(&this->expire_time, 1);
    };
    // 硬件序列号
    std::string hard_id;
    // 开发商ID
    std::string vendor_id;
    // 客户信息
    std::string customer_info;
    // 过期时间
    GDate expire_time;
    // 设备类型
    UKeyPresenceType presence_type;
    // ukey激活码
    std::string ukey_license_code;
    // 每个形态的点数
    UKeyPoints points[UKEY_POINTS_INDEX_LAST];
};

#define INSTALL_TYPE_KEYWOARD_DESKTOP "desktop"
#define INSTALL_TYPE_KEYWOARD_SERVER "server"
#define INSTALL_TYPE_KEYWOARD_INDUSTRIAL "industrial"
#define INSTALL_TYPE_KEYWOARD_DEVELOPMENT "development"

#define PIN_SUFFIX ".kylinos"

#define DB_KEY "KylinSecDBKey"

#define CONNECTION(text1, text2) text1##text2
#define CONNECT(text1, text2) CONNECTION(text1, text2)

class Defer
{
public:
    Defer(std::function<void(std::string)> func, std::string fun_name) : func_(func),
                                                                         fun_name_(fun_name) {}
    ~Defer() { func_(fun_name_); }

private:
    std::function<void(std::string)> func_;
    std::string fun_name_;
};

// helper macro for Defer class
#define SCOPE_EXIT(block) Defer CONNECT(_defer_, __LINE__)([&](std::string _arg_function) block, __FUNCTION__)

#define RETURN_VAL_IF_FALSE(cond, val) \
    {                                  \
        if (!(cond)) return val;       \
    }

#define RETURN_VAL_IF_TRUE(cond, val) \
    {                                 \
        if (cond) return val;         \
    }

#define RETURN_VAL_IF_ERROR(cond)                    \
    {                                                \
        LicenseErrorCode retval = cond;              \
        if (retval != LicenseErrorCode::LEC_SUCCESS) \
        {                                            \
            return retval;                           \
        }                                            \
    }

#define RETURN_IF_FALSE(cond) \
    {                         \
        if (!(cond)) return;  \
    }

#define RETURN_IF_TRUE(cond) \
    {                        \
        if (cond) return;    \
    }

#define CONTINUE_IF_FALSE(cond) \
    {                           \
        if (!(cond)) continue;  \
    }

#define CONTINUE_IF_TRUE(cond) \
    {                          \
        if (cond) continue;    \
    }

#define BREAK_IF_FALSE(cond) \
    {                        \
        if (!(cond)) break;  \
    }

#define BREAK_IF_TRUE(cond) \
    {                       \
        if (cond) break;    \
    }

#define BREAK_AND_ERR_IF_FALSE(expr, err, fmt2, ...) \
    {                                                \
        if (!(expr))                                 \
        {                                            \
            err = fmt::format(fmt2, ##__VA_ARGS__);  \
            break;                                   \
        }                                            \
    }
#define BREAK_AND_ERR_IF_TRUE(expr, err, fmt2, ...) \
    {                                               \
        if (expr)                                   \
        {                                           \
            err = fmt::format(fmt2, ##__VA_ARGS__); \
            break;                                  \
        }                                           \
    }

#define RETURN_VAL_AND_ERR(val, err, fmt2, ...) \
    {                                           \
        err = fmt::format(fmt2, ##__VA_ARGS__); \
        return val;                             \
    }

#define RETURN_VAL_AND_ERR_IF_FALSE(expr, val, err, fmt2, ...) \
    {                                                          \
        if (!(expr))                                           \
        {                                                      \
            err = fmt::format(fmt2, ##__VA_ARGS__);            \
            return val;                                        \
        }                                                      \
    }

#define RETURN_VAL_AND_ERR_IF_TRUE(expr, val, err, fmt2, ...) \
    {                                                         \
        if (expr)                                             \
        {                                                     \
            err = fmt::format(fmt2, ##__VA_ARGS__);           \
            return val;                                       \
        }                                                     \
    }

#define RETURN_VAL_AND_WARN_IF_FALSE(expr, val, fmt2, ...) \
    {                                                      \
        if (!expr)                                         \
        {                                                  \
            KLOG_WARNING(fmt2, ##__VA_ARGS__);             \
            return val;                                    \
        }                                                  \
    }

#define RETURN_VAL_AND_WARN_IF_TRUE(expr, val, fmt2, ...) \
    {                                                     \
        if (expr)                                         \
        {                                                 \
            KLOG_WARNING(fmt2, ##__VA_ARGS__);            \
            return val;                                   \
        }                                                 \
    }

#define IGNORE_EXCEPTION(expr)       \
    {                                \
        try                          \
        {                            \
            expr;                    \
        }                            \
        catch (const Glib::Error &e) \
        {                            \
        }                            \
    }

#define POINTER_TO_STRING(p) ((p) ? p : std::string())

}  // namespace Kiran
