/**
 * @file          /kylin-license/lib/base/license-error.cpp
 * @brief         
 * @author        tangjie02 <tangjie02@kylinos.com.cn>
 * @copyright (c) 2020 KylinSec. All rights reserved. 
 */

#include "lib/base/license-error.h"

#include <fmt/format.h>
#include <glib/gi18n.h>

namespace Kiran
{
LicenseError::LicenseError()
{
}

std::string LicenseError::get_error_desc(LicenseErrorCode error_code)
{
    switch (error_code)
    {
    case LicenseErrorCode::LEC_COMMON_UNSUPPORTED_OPERATION:
        return fmt::format(_("Unsupported operation. (error code: 0x{:x})"), error_code);
        break;
    case LicenseErrorCode::LEC_COMMON_IO_ERROR:
        return fmt::format(_("IO Error. (error code: 0x{:x})"), error_code);
        break;
    case LicenseErrorCode::LEC_LICENSE_LENGTH_INVALID:
    case LicenseErrorCode::LEC_LICENSE_TRANS_FAILED:
    case LicenseErrorCode::LEC_LICENSE_DECODE_FAILED:
    case LicenseErrorCode::LEC_LICENSE_DATE_INVALID:
    case LicenseErrorCode::LEC_UNMATCH_MACHINE_CODE:
        return fmt::format(_("Activation code error. (error code: 0x{:x})"), error_code);
    case LicenseErrorCode::LEC_INSTALL_TYPE_UNMATCH_MACHINE:
        return fmt::format(_("The activation code does not match the machine. (error code: 0x{:x})"), error_code);
    case LicenseErrorCode::LEC_LICENSE_EXPIRED:
        return fmt::format(_("The activation code is expired. (error code: 0x{:x})"), error_code);
    case LicenseErrorCode::LEC_MACHINE_CODE_TRANS_FAILED:
        return fmt::format(_("Machine code error. (error code: 0x{:x})"), error_code);
    case LicenseErrorCode::LEC_SYNC_TO_FILE_FAILED:
        return fmt::format(_("Failed to write activation code. (error code: 0x{:x})"), error_code);
    case LicenseErrorCode::MACHINE_CODE_UNMATCH_INSTALL_TYPE:
        return fmt::format(_("The install class does not match the machine code.(error code: 0x{:x})"), error_code);
    case LicenseErrorCode::LEC_EXPIRED_DATE_LARGE:
    case LicenseErrorCode::LEC_UKEY_GROUP_UNINIT:
    case LicenseErrorCode::LEC_UKEY_EXPIRED_DATE_INVALIDE:
    case LicenseErrorCode::LEC_ENCODE_GENERATE_LICENSE_FAILED:
    case LicenseErrorCode::LEC_GENERATE_LICENSE_LENGTH_INALID:
    case LicenseErrorCode::LEC_SYNC_AUTH_TO_DB_FAILED:
    case LicenseErrorCode::LEC_SYNC_UKEY_TO_DB_FAILED:
    case LicenseErrorCode::LEC_UKEY_ALREADY_EXIST_MACHINE_CODE:
    case LicenseErrorCode::LEC_UKEY_NO_REMAINING_POINTS_2:
    case LicenseErrorCode::LEC_UKEY_NOT_EXIST_MACHINE_CODE:
    case LicenseErrorCode::LEC_UKEY_NO_USED_POINTS:
    case LicenseErrorCode::LEC_C51_GENERATE_LICENSE_FAILED:
    case LicenseErrorCode::LEC_UKEY_ITEM_EXCEED_LIMIT:
    case LicenseErrorCode::LEC_UKEY_PIN_FAILED:
    case LicenseErrorCode::LEC_UKEY_WRITE_MACHINE_CODE_FAILED:
    case LicenseErrorCode::LEC_UKEY_WRITE_REMAINING_POINTS_FAILED:
        return fmt::format(_("Data error. (error code: 0x{:x})"), error_code);
    case LicenseErrorCode::LEC_UKEY_NOT_FOUND:
        return fmt::format(_("USBKey device is not found, please insert USBKey device. (error code: 0x{:x})"), error_code);
    case LicenseErrorCode::LEC_UKEY_NOT_USABLE_UKEY_DEVICE:
        return fmt::format(_("There is no USBKey device available. Please check the expiration date and remaining points of USBKey device. (error code: 0x{:x})"), error_code);
    case LicenseErrorCode::LEC_UKEY_UNSUPPORTED_UNX86:
        return fmt::format(_("The device is not supported by the system architecture. (error code: 0x{:x})"), error_code);
    case LicenseErrorCode::LEC_CONNECT_SERVER_FAILED:
        return fmt::format(_("Failed to connect license server. (error code: 0x{:x})"), error_code);
    case LicenseErrorCode::LEC_SEND_REQUEST_FAILED:
        return fmt::format(_("Failed to send register request. (error code: 0x{:x})"), error_code);
    case LicenseErrorCode::LEC_SERVER_RESPONSE_TIMEOUT:
        return fmt::format(_("Server response timeout. (error code: 0x{:x})"), error_code);
    case LicenseErrorCode::LEC_SERVER_RESPONSE_ERROR:
        return fmt::format(_("Server response error. (error code: 0x{:x})"), error_code);
    case LicenseErrorCode::LEC_INPUT_DATE_INVALID:
    case LicenseErrorCode::LEC_INPUT_CUSTOMER_LENGTH_INVALID:
        return fmt::format(_("Input error. (error code: 0x{:x})"), error_code);
    default:
        return fmt::format(_("Unknown error. (error code: 0x{:x})"), error_code);
    }
}
}  // namespace Kiran