/**
 * @file          /kylin-license/lib/base/license-error.h
 * @brief         
 * @author        tangjie02 <tangjie02@kylinos.com.cn>
 * @copyright (c) 2020 KylinSec. All rights reserved. 
 */

#pragma once

#include <string>

#define LICENSE_NEW_INTERFACE
#include "license_i.h"

namespace Kiran
{
#define LICENSE_ERROR2STR(error_code) LicenseError::get_error_desc(error_code)

#define THROW_DBUSCXX_ERROR(err_message, error_code, ...)                              \
    {                                                                                  \
        err_message = fmt::format(LICENSE_ERROR2STR(error_code), ##__VA_ARGS__);       \
        throw ::DBus::Error("org.freedesktop.DBus.Error.Failed", err_message.c_str()); \
    }

#define DBUS_ERROR_REPLY(error_code, ...)                                                    \
    {                                                                                        \
        auto err_message = fmt::format(LICENSE_ERROR2STR(error_code), ##__VA_ARGS__);        \
        invocation.ret(Glib::Error(G_DBUS_ERROR, G_DBUS_ERROR_FAILED, err_message.c_str())); \
    }

#define DBUS_ERROR_REPLY_AND_RET(error_code, ...) \
    DBUS_ERROR_REPLY(error_code, ##__VA_ARGS__);  \
    return;

class LicenseError
{
public:
    LicenseError();
    virtual ~LicenseError(){};

    static std::string get_error_desc(LicenseErrorCode error_code);
};
}  // namespace Kiran
