/**
 * @file          /kylin-license/lib/base/license-utils.h
 * @brief         
 * @author        tangjie02 <tangjie02@kylinos.com.cn>
 * @copyright (c) 2020 KylinSec. All rights reserved. 
 */

#pragma once

#include <giomm.h>
#include <memory>

#define LICENSE_NEW_INTERFACE
#include "license_i.h"

namespace Kiran
{
struct ActivationCodeFields
{
    std::string machine_code;
    GDate expired_date;
    std::string customer_info;
};

class LicenseUtils
{
public:
    LicenseUtils(){};
    virtual ~LicenseUtils(){};

    // 判断是否为云桌面
    static bool is_ksvd();

    // 判断是否为虚拟机
    static bool is_virtual_machine();

    // 是否未本机
    static bool is_localhost(const std::string &host);

    // 将激活码的二进制形式转换为字符串格式(xxxxx-xxxxx-xxxxx-xxxxx-xxxxx)
    static std::string format_activation_code(const std::string &activation_code);
    // 将激活码的二进制形式转换为旧的字符串格式(xxxx-xxxx-xxxx-xxxx-xxxx-xxxx-xxxx-xxxx)
    static std::string format_long_activation_code(const std::string &activation_code);
    // 将激活码的字符串格式((xxxxx-xxxxx-xxxxx-xxxxx-xxxxx)转换为二进制形式
    static LicenseErrorCode unformat_activation_code(const std::string &str, std::string &activation_code);

    // 将机器码的二进制形式转换为字符串格式(xxxx-xxxx)
    static std::string format_machine_code(const std::string &machine_code);
    // 将机器码的字符串格式(xxxx-xxxx)转换为二进制形式
    static LicenseErrorCode unformat_machine_code(const std::string &str, std::string &machine_code);

    // 根据名称生成密钥
    static std::string generate_key(const std::string &name);

    // 解析激活码得到机器码、过期时间和客户代码
    static std::shared_ptr<ActivationCodeFields> parse_activation_code(const std::string &license_code_str,
                                                                       const std::string &key,
                                                                       LicenseErrorCode &error_code);

private:
    static std::string binary_to_code(const unsigned char *in_binary, int binary_len);
    static bool code_to_binary(const char *in_str, unsigned char out_binary[], int out_len);

    static std::string unformat_license_show_code(std::string str);
};
}  // namespace Kiran
