/**
 * @file          /kylin-license/lib/base/time-utils.cpp
 * @brief         
 * @author        tangjie02 <tangjie02@kylinos.com.cn>
 * @copyright (c) 2020 KylinSec. All rights reserved. 
 */

#include "lib/base/time-utils.h"
#include "lib/base/license-common.h"

namespace Kiran
{
GDate TimeUtils::get_current_gdate()
{
    GDate current_date;
    GTimeVal tv;

    g_date_clear(&current_date, 1);
    g_get_current_time(&tv);
    g_date_set_time_val(&current_date, &tv);
    return current_date;
}

time_t TimeUtils::gdate2timestamp(const GDate *gdate)
{
    struct tm tm_time;

    if (!g_date_valid(gdate))
    {
        return 0;
    }
    g_date_to_struct_tm(gdate, &tm_time);
    return mktime(&tm_time);
}

time_t TimeUtils::str2timestamp(const char *str)
{
    GDateTime *gdatetime = NULL;
    gint year = 0;
    gint month = 0;
    gint day = 0;
    gint hour = 0;
    gint minute = 0;
    gint seconds = 0;
    time_t retval = time(NULL);

    RETURN_VAL_IF_TRUE(str == NULL, retval);

    if (sscanf(str, "%d-%d-%d %d:%d:%d", &year, &month, &day, &hour, &minute, &seconds) >= 3)
    {
	    gdatetime = g_date_time_new_local(year, month, day, hour, minute, seconds);
	    retval = (time_t)g_date_time_to_unix(gdatetime);
    }

    if (gdatetime)
    {
	    g_date_time_unref(gdatetime);
    }
    return  retval;
}

GDate TimeUtils::timestamp2gdate(time_t t)
{
    GDate result;
    struct tm *tm_time = NULL;

    tm_time = localtime(&t);
    g_date_clear(&result, 1);
    g_date_set_dmy(&result,
                   GDateDay(tm_time->tm_mday),
                   GDateMonth(tm_time->tm_mon + 1),
                   GDateYear(tm_time->tm_year + 1900));
    return result;
}

std::string TimeUtils::timestamp2str(time_t t)
{
    struct tm *tm_time;

    RETURN_VAL_IF_TRUE(t == 0, std::string());
    tm_time = localtime(&t);
    return TimeUtils::tm2str(tm_time);
}

std::string TimeUtils::gdate2str(const GDate *date)
{
	gchar *str = g_strdup_printf("%04d-%02d-%02d",
			g_date_get_year(date),
			g_date_get_month(date),
			g_date_get_day(date));

	std::string result = std::string(str);
        g_free(str);
	return result;
}

char *TimeUtils::gdatetime2str(GDateTime *datetime)
{
    return g_strdup_printf("%04d-%02d-%02d %02d:%02d:%02d",
                           g_date_time_get_year(datetime),
                           g_date_time_get_month(datetime),
                           g_date_time_get_day_of_month(datetime),
                           g_date_time_get_hour(datetime),
                           g_date_time_get_minute(datetime),
                           g_date_time_get_second(datetime));
}

std::string TimeUtils::tm2str(const struct tm *tm_time)
{
    gchar *str = g_strdup_printf("%04d-%02d-%02d %02d:%02d:%02d",
                                            tm_time->tm_year + 1900,
                                            tm_time->tm_mon + 1,
                                            tm_time->tm_mday,
                                            tm_time->tm_hour,
                                            tm_time->tm_min,
                                            tm_time->tm_sec);
    std::string result = std::string(str);
    g_free(str);
    return result;
}

bool TimeUtils::arr2gdate(const unsigned char dt[], GDate *g_date)
{
    char *date_str = NULL;

    date_str = g_strdup_printf("%4d-%02d-%02d", dt[0] + 2000, dt[1], dt[2]);
    g_date_set_parse(g_date, date_str);
    g_free(date_str);
    return g_date_valid(g_date);
}

LicenseErrorCode TimeUtils::gdate2arr(const GDate *gd, unsigned char *ld)
{
    unsigned int year, month, day;

    year = g_date_get_year(gd);
    month = g_date_get_month(gd);
    day = g_date_get_day(gd);

    if (year >= 2256)
    {
        return LicenseErrorCode::LEC_EXPIRED_DATE_LARGE;
    }

    ld[0] = year - 2000;
    ld[1] = month;
    ld[2] = day;

    return LicenseErrorCode::LEC_SUCCESS;
}
}  // namespace Kiran
