/**
 * @file          /kylin-license/lib/ukey/ukey-database.h
 * @brief         
 * @author        tangjie02 <tangjie02@kylinos.com.cn>
 * @copyright (c) 2020 KylinSec. All rights reserved. 
 */

#pragma once

#include <sqlcipher/sqlite3.h>

#include <map>
#include <vector>

#include "lib/base/license-common.h"

namespace Kiran
{
struct DBAuthorizedItem
{
    std::string machine_code;
    std::string license_code;
    std::string expired_date;
    std::string activation_time;
    std::string hard_id;
};

typedef std::vector<std::shared_ptr<DBAuthorizedItem>> DBAuthorizedVec;

struct DBUKeyItem
{
    std::string hard_id;
    std::string vendor_id;
    std::string customer_info;
    std::string expired_date;
    int32_t presence_type;
    std::string ukey_license_code;
    int32_t almighty_authorized_points;
    int32_t almighty_remaining_points;
    int32_t desktop_authorized_points;
    int32_t desktop_remaining_points;
    int32_t server_authorized_points;
    int32_t server_remaining_points;
    int32_t industrial_authorized_points;
    int32_t industrial_remaining_points;
    int32_t development_authorized_points;
    int32_t development_remaining_points;
};

typedef std::vector<std::shared_ptr<DBUKeyItem>> DBUKeyVec;

class UKeyDatabase
{
public:
    UKeyDatabase();
    virtual ~UKeyDatabase();

    void init();

    DBUKeyVec get_ukeys(const std::string &vendor_id, int32_t presence_type);
    std::shared_ptr<DBUKeyItem> get_ukey_item(const std::string &hard_id);

    // 如果对应ukey不存在则添加，否则替换之前的ukey
    bool set_ukey_item(std::shared_ptr<DBUKeyItem> ukey_item);

    DBAuthorizedVec get_authorized_list(const std::string &hard_id);

    bool set_authorized_item(std::shared_ptr<DBAuthorizedItem> authorized_item);

private:
    // 删除物理ukey设备相关数据
    bool clear_physical_ukeys(std::string &error);

    // 加载数据库
    bool load(std::string &error);

    static int parse_authorized_item(void *user_data, int column_num, char **values, char **names);
    static int parse_ukey_item(void *user_data, int column_num, char **values, char **names);

    bool exec_sql(std::string sql, std::string &error, sqlite3_callback callback = NULL, void *user_data = NULL);

private:
    std::string file_path_;
    sqlite3 *db_handle_;

    // 已授权信息表 <硬件序列号，已授权机器列表>
    std::map<std::string, DBAuthorizedVec> authorized_table_;
    // ukey设备信息表
    std::map<std::string, std::shared_ptr<DBUKeyItem>> ukey_table_;
};

}  // namespace Kiran
