/**
 * @file          /kylin-license/lib/ukey/ukey-device.h
 * @brief         
 * @author        tangjie02 <tangjie02@kylinos.com.cn>
 * @copyright (c) 2020 KylinSec. All rights reserved. 
 */

#pragma once

#include <set>

#include "lib/base/base.h"

namespace Kiran
{
#define PIN_CODE_MAX_LEN 100

class UKeyDevice
{
public:
    UKeyDevice(const std::string &vendor_id);
    virtual ~UKeyDevice(){};

    // 设备存在类型
    virtual UKeyPresenceType get_presence_type() = 0;

    // 通过机器码生成授权码
    virtual LicenseErrorCode generate_license_code(const std::string &machine_code,
                                                   const std::string &key,
                                                   std::string &out_license_code) = 0;

    virtual bool update(std::string &error) = 0;

    // 打印设备信息
    virtual void print_ukey();

    // 获取硬件序列号
    std::string get_hard_id() { return this->hard_id_; };

    // 获取高级存储区
    const UKeyHighAddress &get_high_address_area() { return this->high_address_area_; }

    // UKEY是否已经过期
    bool is_expired();

    // 判断机器码是否已经授权
    bool exist_machine_code(const std::string &machine_code);

    // 判断指定安装类型是否存在剩余点数
    bool has_remaining_points(int32_t install_type);

protected:
    // 获取已使用点数
    int32_t get_used_points();

    // 获取对应安装形态的剩余点数的索引
    UKeyPointsIndex get_remaining_points_index(int32_t install_type);

    // 获取对应安装形态已使用点数的索引
    UKeyPointsIndex get_used_points_index(int32_t install_type);

    // 添加授权信息，减少剩余点数
    LicenseErrorCode add_authorized_info(const std::string &machine_code,
                                         UKeyPointsIndex &out_points_index);

    // 删除授权信息，增加剩余点数
    LicenseErrorCode del_authorized_info(const std::string &machine_code,
                                         UKeyPointsIndex points_index);

    // 同步到数据库
    LicenseErrorCode sync_authorized_to_db(const std::string &machine_code,
                                           const std::string &activation_code,
                                           const GDate &expired_date);

protected:
    // 硬件序列号
    std::string hard_id_;
    // 开发商ID
    std::string vendor_id_;
    // 开发商口令
    std::string pincode_;
    // 高级存储区数据
    UKeyHighAddress high_address_area_;
    // 已授权的机器码
    std::set<std::string> machine_codes_;
};

typedef std::vector<std::shared_ptr<UKeyDevice>> UKeyDeviceVec;

}  // namespace Kiran
