/**
 * @file          /kylin-license/lib/ukey/ukey-group.h
 * @brief         
 * @author        tangjie02 <tangjie02@kylinos.com.cn>
 * @copyright (c) 2020 KylinSec. All rights reserved. 
 */

#pragma once

#include <giomm.h>

#include <set>

#include "lib/base/base.h"
#include "lib/ukey/ukey-device.h"

namespace Kiran
{
// 开发商ID相同的USBKey设备归为同一个USBKey组
class UKeyGroup
{
public:
    UKeyGroup(const std::string &vendor_id, UKeyPresenceType ukey_presence_type);
    virtual ~UKeyGroup();

    // 通过机器码生成授权码
    LicenseErrorCode generate_license_code(const std::string &machine_code,
                                           const std::string &key,
                                           std::shared_ptr<UKeyDevice> &out_ukey_device,
                                           std::string &out_license_code);

    // 获取已对machine_code进行授权的ukey设备，优先选择物理ukey设备，如果存在多个，则选择过期时间最长的一个
    std::shared_ptr<UKeyDevice> get_ukey_by_machine_code(const std::string &machine_code);
    // 获取所有UKey设备数据
    UKeyDeviceVec get_ukeys();

    /* 如下情况会导致ukey数据发生变化：
    1. ukey设备插拔
    2. 其他进程对ukey进行了写操作
    3. 当前进程对ukey进行了写操作 */
    // sigc::signal<void> signal_ukey_changed() { return this->ukey_changed_; };

private:
    // 加载USBKey设备组
    void load_ukeys();

    void load_virtual_ukeys();

    void load_physical_ukeys();
    void update_physical_ukeys();
    void clear_physical_ukeys();

    void add_ukey(std::shared_ptr<UKeyDevice> ukey_device);

    // 发现还存在剩余点数的USBKey设备
    std::shared_ptr<UKeyDevice> find_usable_ukey(int32_t install_type);

    // 这里使用空指针，避免在头文件中包含gudev.h文件
    static void on_uevent_changed(void *client, gchar *action, void *device, gpointer user_data);

    void on_rockey_file_changed(const Glib::RefPtr<Gio::File> &file, const Glib::RefPtr<Gio::File> &other_file, Gio::FileMonitorEvent event_type);

private:
    // 设备厂商ID
    std::string vendor_id_;
    // 加载的ukey类型
    UKeyPresenceType ukey_presence_type_;
    // <硬件序列号, USBKey数据>
    std::map<std::string, std::shared_ptr<UKeyDevice>> ukeys_;
    // <机器码,对应的ukey设备列表>
    std::map<std::string, UKeyDeviceVec> machine_codes_;
    // 这里也使用空指针
    void *udev_client_;
    uint64_t udev_client_handler_;
    // 监控ROCKEY更新
    Glib::RefPtr<Gio::FileMonitor> rockey_update_monitor_;
};

}  // namespace Kiran