/**
 * @file          /kylin-license/lib/ukey/ukey-manager.cpp
 * @brief         
 * @author        tangjie02 <tangjie02@kylinos.com.cn>
 * @copyright (c) 2020 KylinSec. All rights reserved. 
 */

#include "lib/ukey/ukey-manager.h"

namespace Kiran
{
UKeyManager::UKeyManager(UKeyPresenceType ukey_presence_type) : ukey_presence_type_(ukey_presence_type)
{
    this->ukey_database_ = std::make_shared<UKeyDatabase>();
}

UKeyManager::~UKeyManager()
{
}

UKeyManager* UKeyManager::instance_ = NULL;
void UKeyManager::global_init(UKeyPresenceType ukey_presence_type)
{
    instance_ = new UKeyManager(ukey_presence_type);
    instance_->init();
}

void UKeyManager::init()
{
    this->ukey_database_->init();
}

std::shared_ptr<UKeyGroup> UKeyManager::create_ukey_group(const std::string& vendor_id)
{
    auto iter = this->ukey_groups_.find(vendor_id);
    if (iter != this->ukey_groups_.end())
    {
        return iter->second;
    }

    auto ukey_group = std::make_shared<UKeyGroup>(vendor_id, this->ukey_presence_type_);
    this->ukey_groups_[vendor_id] = ukey_group;
    return ukey_group;
}

std::shared_ptr<UKeyGroup> UKeyManager::get_ukey_group(const std::string& vendor_id)
{
    auto iter = this->ukey_groups_.find(vendor_id);
    if (iter == this->ukey_groups_.end())
    {
        return std::shared_ptr<UKeyGroup>();
    }
    return iter->second;
}

}  // namespace Kiran
