/**
 * @file          /kylin-license/lib/ukey/ukey-manager.h
 * @brief         
 * @author        tangjie02 <tangjie02@kylinos.com.cn>
 * @copyright (c) 2020 KylinSec. All rights reserved. 
 */

#include "lib/ukey/ukey-database.h"
#include "lib/ukey/ukey-group.h"

namespace Kiran
{
class UKeyManager
{
public:
    virtual ~UKeyManager();

    static UKeyManager* get_instance() { return instance_; };

    static void global_init(UKeyPresenceType ukey_presence_type);

    static void global_deinit() { delete instance_; };

    // 如果不存在则进行创建，否则直接返回已存在的UKeyGroup
    std::shared_ptr<UKeyGroup> create_ukey_group(const std::string& vendor_id);

    // 如果不存在则返回nullptr
    std::shared_ptr<UKeyGroup> get_ukey_group(const std::string& vendor_id);

    // 获取虚拟ukey对应的数据库对象
    std::shared_ptr<UKeyDatabase> get_ukey_database() { return this->ukey_database_; }

private:
    UKeyManager(UKeyPresenceType ukey_presence_type);

    void init();

private:
    static UKeyManager* instance_;
    // 虚拟ukey对应的数据库
    std::shared_ptr<UKeyDatabase> ukey_database_;
    // 加载的ukey类型
    UKeyPresenceType ukey_presence_type_;
    // <开发商ID，相同开发商ID的ukey组>
    std::map<std::string, std::shared_ptr<UKeyGroup>> ukey_groups_;
};

}  // namespace Kiran