/**
 * @file          /kylin-license/lib/ukey/ukey-physical-device.h
 * @brief         
 * @author        tangjie02 <tangjie02@kylinos.com.cn>
 * @copyright (c) 2020 KylinSec. All rights reserved. 
 */

#pragma once

#include "lib/ukey/ukey-device.h"

#if defined(__x86_64__) && defined(ENABLE_ROCKEY)
#include "RY3_API.h"
#else
typedef void *RY_HANDLE;
#endif

namespace Kiran
{
// 机器码校验位长度
#define MACHINE_CODE_CHECK_BIT_LEN 2
// Ukey存储机器码使用的长度 (机器码长度+校验位长度)
#define UKEY_MACHINE_CODE_ITEM_LEN (MACHINE_CODE_LENGTH + MACHINE_CODE_CHECK_BIT_LEN)

// UKey低级存储区每项数据结构
struct UKeyLowAddressItem
{
    std::string machine_code;
    //两字节占位符,作为校验码
    unsigned char placeholder[MACHINE_CODE_CHECK_BIT_LEN];
};

// ukey 低级存储区数据结构
struct UKeyLowAddress
{
    UKeyLowAddress();
    UKeyLowAddressItem item[UKEY_MAX_AUTHORIZED_NUM];
    gint32 item_count;
};

class UKeyPhysicalDevice : public UKeyDevice
{
public:
    UKeyPhysicalDevice(const std::string &vendor_id, RY_HANDLE handle);
    virtual ~UKeyPhysicalDevice();

    bool init();

    virtual UKeyPresenceType get_presence_type() { return UKeyPresenceType::UKEY_PRESENCE_TYPE_PHYSICAL; };

    virtual LicenseErrorCode generate_license_code(const std::string &machine_code,
                                                   const std::string &key,
                                                   std::string &out_license_code);

    // ukey设备被其他程序修改时需要更新
    virtual bool update(std::string &error);

    virtual void print_ukey();

private:
    // 加载USBKey设备中的数据
    bool load_ukey(std::string &error);
    // 获取高级存储区数据
    bool load_high_address_area(std::string &error);
    // 获取低级存储区数据(UsbKey中存放的已授权信息)
    bool load_low_address_area(int32_t start, int32_t len, std::string &error);

    LicenseErrorCode write_low_address_item(UKeyLowAddressItem &low_address_item);

    // 通过索引获取剩余点数偏移位置
    UKeyAddressOffset get_remaining_points_offset(UKeyPointsIndex index);

private:
    // 设备句柄
    RY_HANDLE handle_;
    // 低级存储区数据
    UKeyLowAddress low_address_area_;
};

}  // namespace Kiran