/**
 * @file          /kylin-license/src/daemon/daemon.cpp
 * @brief         
 * @author        tangjie02 <tangjie02@kylinos.com.cn>
 * @copyright (c) 2020 KylinSec. All rights reserved. 
 */

#include "src/daemon/daemon.h"

#include <dbus-c++/glib-integration.h>

namespace Kiran
{
namespace Daemon
{
DBus::Glib::BusDispatcher dispatcher;

Daemon::Daemon()
{
    this->configuration_ = std::make_shared<LicenseConfiguration>();
    this->machine_ = std::make_shared<LicenseMachine>();

    try
    {
        DBus::default_dispatcher = &dispatcher;
        dispatcher.attach(NULL);

        DBus::Connection connection = DBus::Connection::SystemBus();
        connection.request_name(LICENSE_MANAGER_DBUS_NAME);
        connection.request_name(LICENSE_HELPER_DBUS_NAME);
        this->dbus_license_manager_ = std::make_shared<DBusLicenseManager>(connection);
    }
    catch (const DBus::Error &e)
    {
        KLOG_WARNING("%s: %s.", e.name(), e.message());
    }
}

Daemon::~Daemon()
{
}

Daemon *Daemon::instance_ = NULL;
void Daemon::global_init()
{
    instance_ = new Daemon();
    instance_->init();
}

std::shared_ptr<LicenseObject> Daemon::find_license_object(const std::string &name)
{
    auto license_object = this->get_license_object(name);
    // 如果已经存在了，则直接返回
    RETURN_VAL_IF_TRUE(license_object, license_object);

    auto license_object_group = this->configuration_->get_license_object_group(name);

    if (!license_object_group)
    {
        license_object_group = std::make_shared<LicenseObjectGroup>(name);
        this->configuration_->set_license_object_group(license_object_group);
    }

    // 添加新的授权对象
    return this->add_license_object(name);
}

void Daemon::init()
{
    this->configuration_->init();
    this->machine_->init();
    this->load_license_objects();
    this->dbus_license_manager_->init();
}

void Daemon::load_license_objects()
{
    auto license_object_groups = this->configuration_->get_license_object_groups();

    for (uint32_t i = 0; i < license_object_groups.size(); ++i)
    {
        auto &license_object_group = license_object_groups[i];
        this->add_license_object(license_object_group->name);
    }
}

std::shared_ptr<LicenseObject> Daemon::add_license_object(const std::string &name)
{
    KLOG_PROFILE("Name: %s.", name.c_str());

    std::shared_ptr<LicenseObject> license_object;
    license_object = std::make_shared<LicenseObject>(name);
    license_object->init();
    this->license_objects_[name] = license_object;
    return license_object;
}

}  // namespace Daemon

}  // namespace Kiran
