/**
 * @file          /kylin-license/src/daemon/daemon.h
 * @brief         
 * @author        tangjie02 <tangjie02@kylinos.com.cn>
 * @copyright (c) 2020 KylinSec. All rights reserved. 
 */

#pragma once

#include "src/daemon/dbus/dbus-license-manager.h"
#include "src/daemon/license-configuration.h"
#include "src/daemon/license-machine.h"
#include "src/daemon/license-object.h"

namespace Kiran
{
namespace Daemon
{
class Daemon
{
public:
    Daemon();
    virtual ~Daemon();

    static Daemon *get_instance() { return instance_; };

    static void global_init();

    static void global_deinit() { delete instance_; };

    // 获取配置
    std::shared_ptr<LicenseConfiguration> get_configuration() { return this->configuration_; };
    // 获取机器信息
    std::shared_ptr<LicenseMachine> get_machine() { return this->machine_; };
    // 获取DBus管理类
    std::shared_ptr<DBusLicenseManager> get_dbus_license_manager() { return this->dbus_license_manager_; }

    // 获取/设置授权对象的信息
    LicenseObjectVec get_license_objects() { return MapHelper::get_values(this->license_objects_); };
    std::shared_ptr<LicenseObject> get_license_object(const std::string &name) { return MapHelper::get_value(this->license_objects_, name); };
    // 如果不存在则创建新的授权对象，返回新的或者已经创建的对象
    std::shared_ptr<LicenseObject> find_license_object(const std::string &name);

private:
    void init();

    void load_license_objects();
    std::shared_ptr<LicenseObject> add_license_object(const std::string &name);

private:
    static Daemon *instance_;
    // 授权DBUS管理
    std::shared_ptr<DBusLicenseManager> dbus_license_manager_;
    // 相关配置
    std::shared_ptr<LicenseConfiguration> configuration_;
    std::shared_ptr<LicenseMachine> machine_;
    // 所有授权对象
    std::map<std::string, std::shared_ptr<LicenseObject>> license_objects_;
};
}  // namespace Daemon
}  // namespace Kiran