/**
 * @file          /kylin-license/src/daemon/dbus/dbus-license-manager.cpp
 * @brief         
 * @author        tangjie02 <tangjie02@kylinos.com.cn>
 * @copyright (c) 2020 KylinSec. All rights reserved. 
 */

#include "src/daemon/dbus/dbus-license-manager.h"
#include "src/daemon/daemon.h"
#include "src/daemon/license-configuration.h"

namespace Kiran
{
namespace Daemon
{
DBusLicenseManager::DBusLicenseManager(DBus::Connection &connection) : DBus::ObjectAdaptor(connection, LICENSE_MANAGER_OBJECT_PATH),
                                                                       dbus_connection_(connection)
{
    this->license_helper_ = std::make_shared<DBusLicenseHelper>(connection);
}

DBusLicenseManager::~DBusLicenseManager()
{
}

void DBusLicenseManager::init()
{
    this->license_helper_->init();

    this->load_dbus_license_objects();
}

::DBus::Path DBusLicenseManager::GetLicenseObject(const std::string &name)
{
    std::shared_ptr<DBusLicenseObject> dbus_license_object;

    dbus_license_object = this->get_dbus_license_object(name);
    // 不存在则创建
    if (!dbus_license_object)
    {
        auto license_object = Daemon::get_instance()->find_license_object(name);
        dbus_license_object = this->add_dbus_license_object(license_object);
    }

    if (!dbus_license_object)
    {
        THROW_DBUSCXX_ERROR(this->dbus_error_message_, LicenseErrorCode::LEC_COMMON_NULL_POINTER);
    }
    else
    {
        return ::DBus::Path(dbus_license_object->get_object_path());
    }
}

void DBusLicenseManager::load_dbus_license_objects()
{
    auto license_objects = Daemon::get_instance()->get_license_objects();
    for (uint32_t i = 0; i < license_objects.size(); ++i)
    {
        this->add_dbus_license_object(license_objects[i]);
    }
}

std::shared_ptr<DBusLicenseObject> DBusLicenseManager::add_dbus_license_object(std::shared_ptr<LicenseObject> license_object)
{
    auto dbus_license_object = std::make_shared<DBusLicenseObject>(this->dbus_connection_, license_object);
    dbus_license_object->init();
    this->dbus_license_objects_[license_object->get_name()] = dbus_license_object;
    return dbus_license_object;
}

}  // namespace Daemon

}  // namespace Kiran
