/**
 * @file          /kylin-license/src/daemon/dbus/dbus-license-manager.h
 * @brief         
 * @author        tangjie02 <tangjie02@kylinos.com.cn>
 * @copyright (c) 2020 KylinSec. All rights reserved. 
 */

#pragma once

#include "lib/base/base.h"
#include "license_manager_dbus_stub.h"
#include "src/daemon/dbus/dbus-license-helper.h"
#include "src/daemon/dbus/dbus-license-object.h"

namespace Kiran
{
namespace Daemon
{
class DBusLicenseManager : public com::kylinsec::Kiran::LicenseManager_adaptor,
                           public DBus::IntrospectableAdaptor,
                           public DBus::PropertiesAdaptor,
                           public DBus::ObjectAdaptor
{
public:
    DBusLicenseManager(DBus::Connection &connection);
    virtual ~DBusLicenseManager();

    void init();

    // 获取/设置DBUS授权对象
    std::shared_ptr<DBusLicenseObject> get_dbus_license_object(const std::string &name) { return MapHelper::get_value(this->dbus_license_objects_, name); };

public:
    // 获取授权对象的ObjectPath
    virtual ::DBus::Path GetLicenseObject(const std::string &name);

    // 正常情况下机器码是不应该发生变化的
    // virtual bool machine_code_setHandler(const Glib::ustring &value) { return false; }
    // virtual Glib::ustring machine_code_get();

private:
    void load_dbus_license_objects();
    std::shared_ptr<DBusLicenseObject> add_dbus_license_object(std::shared_ptr<LicenseObject> license_object);

private:
    DBus::Connection dbus_connection_;
    // 兼容以前的系统授权dbus接口
    std::shared_ptr<DBusLicenseHelper> license_helper_;
    // 维护所有授权对象DBUS
    std::map<std::string, std::shared_ptr<DBusLicenseObject>> dbus_license_objects_;
    // 由于传递给DBus::Error的参数必须是不能被立即销毁的，因此将数据放入成员变量存储
    std::string dbus_error_message_;
};

}  // namespace Daemon

}  // namespace Kiran
