/**
 * @file          /kylin-license/src/daemon/dbus/dbus-license-object.h
 * @brief         
 * @author        tangjie02 <tangjie02@kylinos.com.cn>
 * @copyright (c) 2020 KylinSec. All rights reserved. 
 */

#pragma once

#include "lib/base/base.h"
#include "license_object_dbus_stub.h"
#include "src/daemon/license-object.h"

namespace Kiran
{
namespace Daemon
{
class DBusLicenseObject : public com::kylinsec::Kiran::LicenseObject_adaptor,
                          public ::DBus::IntrospectableAdaptor,
                          public ::DBus::ObjectAdaptor
{
public:
    // 这里不使用继承是因为需要兼容老版本的LicenseHelper dbus接口
    DBusLicenseObject(DBus::Connection &connection, std::shared_ptr<LicenseObject> license_object);
    virtual ~DBusLicenseObject();

    void init();

    std::string get_object_path() { return this->object_path_; };

protected:
    // 获取激活信息
    virtual std::string GetLicense();
    // 通过激活码激活
    virtual void ActivateByActivationCode(const std::string &activation_code);
    // 通过UKey设备激活
    virtual std::string ActivateByUsbKey(const std::string &vendor_id);
    // 在线激活
    virtual void ActivateOnLine(const std::string &host);
    // 设置在线服务器地址
    virtual void SetAuthServer(const std::string &host);

    virtual void on_set_property(InterfaceAdaptor &interface, const std::string &property, const DBus::Variant &value);

private:
    void on_license_info_changed();

private:
    // 授权对象
    std::shared_ptr<LicenseObject> license_object_;

    std::string object_path_;
    uint32_t object_register_id_;

    // 由于传递给DBus::Error的参数必须是不能被立即销毁的，因此将数据放入成员变量存储
    std::string dbus_error_message_;
};

}  // namespace Daemon

}  // namespace Kiran
