/**
 * @file          /kylin-license/src/daemon/main.cpp
 * @brief         
 * @author        tangjie02 <tangjie02@kylinos.com.cn>
 * @copyright (c) 2020 KylinSec. All rights reserved. 
 */

#include <giomm.h>
#include <glib/gi18n.h>
#include <locale.h>

#include "lib/ukey/ukey-manager.h"
#include "src/daemon/daemon.h"

// /etc/profile.d/license.sh需要使用，这里用于占位
#define SYSTEM_NOT_ACTIVATED _("System not activated")

int main(int argc, char **argv)
{
    Gio::init();

    setlocale(LC_ALL, "");
    /* 一些函数（例如：g_date_set_parse）会根据LC_TIME来确定日期时间的默认解析顺序，因此LC_TIME最好使用系统配置的环境变量，
    否则可能会导致授权码中的过期时间解析错误。例如，如果我们系统的语言环境为en_AU.utf8，当调用g_date_set_parse(gdate, "2050-01-01")时，
    得到的日期是"2001-01-02"，因为该函数解析日期的顺序变为了日-月-年。*/
    setlocale(LC_TIME, "en_US.utf8");
    bindtextdomain(GETTEXT_DOMAIN, LICENSE_LOCALEDIR);
    bind_textdomain_codeset(GETTEXT_DOMAIN, "UTF-8");
    textdomain(GETTEXT_DOMAIN);

    auto loop = Glib::MainLoop::create();
    klog_gtk3_init(std::string(), "kylinsec-system", "kylin-license", Glib::path_get_basename(argv[0]));

    Kiran::UKeyManager::global_init(Kiran::UKeyPresenceType::UKEY_PRESENCE_TYPE_PHYSICAL);
    Kiran::Daemon::Daemon::global_init();

    loop->run();

    Kiran::Daemon::Daemon::global_deinit();
    Kiran::UKeyManager::global_deinit();
    return 0;
}