/**
 * @file          /kylin-license/src/notify/license-notify.cpp
 * @brief         
 * @author        tangjie02 <tangjie02@kylinos.com.cn>
 * @copyright (c) 2020 KylinSec. All rights reserved. 
 */
#include "src/notify/license-notify.h"

#include <glib/gi18n.h>

#include "lib/base/base.h"

namespace Kiran
{
#define NOTIFY_TIMEOUT (10 * 1000)
#define NOTIFY_ICON_INFO "gtk-dialog-info"
#define NOTIFY_ICON_WARN "gtk-dialog-warning"
#define NOTIFY_ICON_ERROR "gtk-dialog-error"


#define LIBNOTIFY_CHECK_VERSION(major, minor) \
	((LIBNOTIFY_MAJOR > major) || \
	(LIBNOTIFY_MAJOR == major && LIBNOTIFY_MINOR >= minor))

LicenseNotify::LicenseNotify()
{
    if (!notify_init(_("Kylinsec")))
    {
        KLOG_WARNING("%s", _("Failed to init libnotify"));
        return;
    }

#if LIBNOTIFY_CHECK_VERSION(0, 7)
	    this->notify_ = notify_notification_new("", "", ""); 
#else
	    this->notify_ = notify_notification_new("", "", "", NULL ); 
#endif
}

LicenseNotify::~LicenseNotify()
{
	if (this->notify_)
	{
		g_object_unref(this->notify_);
	}
    notify_uninit();
}

LicenseNotify *LicenseNotify::instance_ = NULL;

LicenseNotify *LicenseNotify::get_instance()
{
    if (!instance_)
    {
        instance_ = new LicenseNotify();
    }
    return instance_;
}

void LicenseNotify::info(const char *message)
{
    this->notify_send(message, NOTIFY_ICON_INFO);
}

void LicenseNotify::warn(const char *message)
{
    this->notify_send(message, NOTIFY_ICON_WARN);
}

void LicenseNotify::error(const char *message)
{
    this->notify_send(message, NOTIFY_ICON_ERROR);
}

void LicenseNotify::notify_send(const char *msg, const char *icon)
{
    g_return_if_fail(this->notify_ != NULL);

    notify_notification_update(this->notify_, _("Kylinsec"), msg, icon);
    notify_notification_set_timeout(this->notify_, NOTIFY_TIMEOUT);
    notify_notification_show(this->notify_, NULL);
}

}  // namespace Kiran
