/**
 * @file          /kylin-license/src/register/license-register.h
 * @brief         
 * @author        tangjie02 <tangjie02@kylinos.com.cn>
 * @copyright (c) 2020 KylinSec. All rights reserved. 
 */
#include <json/json.h>

#include "lib/base/base.h"
#include "lib/dbus/license_manager_proxy_wrapper.h"
#include "lib/dbus/license_object_proxy_wrapper.h"

namespace Kiran
{
struct CommandOptions
{
    CommandOptions() : registration(false),
                       show_license(false),
                       show_version(false),
                       show_by_json(false) {}
    // 激活
    bool registration;
    // 激活码
    Glib::ustring license_code;
    Glib::ustring activation_code;
    // 激活服务器
    Glib::ustring server_host;
    // UKEY设备的Vendor ID
    Glib::ustring vendor_id;
    // 显示激活信息
    bool show_license;
    // 显示版本号
    bool show_version;
    // 获取变量值
    Glib::ustring variable_name;
    // 激活对象
    Glib::ustring license_object_name;
    // 是否以json格式显示激活信息
    bool show_by_json;
};

class LicenseRegister
{
public:
    LicenseRegister(::DBus::Connection dbus_connection);
    virtual ~LicenseRegister();

    bool init();

    bool run(int &argc, char **&argv);

private:
    void process_get_variable(const std::string &variable_name);

    bool process_registration(const std::string &activation_code,
                              const std::string &auth_server,
                              const std::string &vendor_id);

    void process_show();

    bool get_license(Json::Value &value, std::string &error);

    bool json_str2value(const std::string &str, Json::Value &value, std::string &error);

    std::string get_activation_type_str(uint32_t activation_type);

private:
    Glib::OptionContext option_context_;
    Glib::OptionGroup option_group_;
    CommandOptions options_;

    ::DBus::Connection dbus_connection_;
    std::shared_ptr<DBus::LicenseManagerProxyWrapper> license_manager_proxy_;
    std::shared_ptr<DBus::LicenseObjectProxyWrapper> license_object_proxy_;
};
}  // namespace Kiran
