/**
 * @file          /kylin-license/src/register/main.cpp
 * @brief         
 * @author        tangjie02 <tangjie02@kylinos.com.cn>
 * @copyright (c) 2020 KylinSec. All rights reserved. 
 */

#include <dbus-c++/glib-integration.h>
#include <glib/gi18n.h>
#include <locale.h>
#include "src/register/license-register.h"

using namespace Kiran;

::DBus::Glib::BusDispatcher dispatcher;

int main(int argc, char* argv[])
{
    setlocale(LC_ALL, "");
    /* 一些函数（例如：g_date_set_parse）会根据LC_TIME来确定日期时间的默认解析顺序，因此LC_TIME最好使用系统配置的环境变量，
    否则可能会导致授权码中的过期时间解析错误。例如，如果我们系统的语言环境为en_AU.utf8，当调用g_date_set_parse(gdate, "2050-01-01")时，
    得到的日期是"2001-01-02"，因为该函数解析日期的顺序变为了日-月-年。*/
    setlocale(LC_TIME, "en_US.utf8");
    bindtextdomain(GETTEXT_DOMAIN, LICENSE_LOCALEDIR);
    bind_textdomain_codeset(GETTEXT_DOMAIN, "UTF-8");
    textdomain(GETTEXT_DOMAIN);

    klog_gtk3_init(std::string(), "kylinsec-session", "kylin-license", Glib::path_get_basename(argv[0]));
    Gio::init();

    try
    {
        ::DBus::default_dispatcher = &dispatcher;
        dispatcher.attach(NULL);
        auto dbus_connection = ::DBus::Connection::SystemBus();
        LicenseRegister license_register(dbus_connection);
        if (!license_register.init())
        {
            return EXIT_FAILURE;
        }

        if (!license_register.run(argc, argv))
        {
            return EXIT_FAILURE;
        }
    }
    catch (const std::exception& e)
    {
        fmt::print(stderr, "{0}", e.what());
        return EXIT_FAILURE;
    }

    return EXIT_SUCCESS;
}
