/**
 * @file          /kylin-license/src/server/socket-manager.h
 * @brief         
 * @author        tangjie02 <tangjie02@kylinos.com.cn>
 * @copyright (c) 2020 KylinSec. All rights reserved. 
 */
#pragma once

#include <giomm.h>
#include <uv.h>

#include "lib/base/base.h"
#include "lib/dbus/license_manager_proxy_wrapper.h"
#include "src/server/license-message.pb.h"

namespace Kiran
{
struct ClientInfo
{
    uv_tcp_t *socket;
    std::string machine_code;
};

class SocketManager
{
public:
    SocketManager();
    virtual ~SocketManager();

    static SocketManager *get_instance() { return instance_; };

    static void global_init();

    static void global_deinit() { delete instance_; };

    void init();

    // 连接建立
    bool add_connection(uv_tcp_t *client);
    // 添加已授权连接
    bool add_auth_connection(uv_tcp_t *client, const std::string &machine_code);
    // 连接断开
    bool del_connection(uv_tcp_t *client);
    // 收到数据
    void read_connection(uv_tcp_t *client, const char *data, size_t data_len);

    uv_os_fd_t get_fd(const std::string &machine_code);
    ClientInfo *get_client_info(const std::string &machine_code);

private:
    bool timing_update_license();
    // 注册请求
    void on_register(uv_tcp_t *client, const KiranLicenseRegisterReq &register_req);
    // 认证请求
    void on_auth(uv_tcp_t *client, const KiranLicenseAuthReq &auth_req);

    // 发送错误消息
    void send_error_msg(uv_tcp_t *client,
                        KiranLicenseProtoId id,
                        KiranLicenseProtoError error_code,
                        const std::string &error_desc);
    // 发送消息
    void send_license_msg(uv_tcp_t *client, const KiranLicenseMessage &msg);

    // 获取可读的IP地址
    std::string get_ip_addr(uv_tcp_t *client);

    uv_os_fd_t get_fd(uv_tcp_t *client);

private:
    static SocketManager *instance_;

    uv_tcp_t socket_server_;

    std::map<uv_os_fd_t, ClientInfo> clients_;
    std::map<std::string, uv_os_fd_t> machines_;

    std::shared_ptr<DBus::LicenseManagerProxyWrapper> proxy_;
    ::DBus::Connection dbus_connection_;

    KiranLicenseMessage message_;
};
}  // namespace Kiran