#include "src/server/uv-glib.h"

#include "lib/base/base.h"

namespace Kiran
{
UVGlib::UVGlib() : main_context_(g_main_context_default()),
                   max_priority_(0),
                   fds_(NULL),
                   allocated_nfds_(0),
                   nfds_(0),
                   poll_handles_(NULL)

{
}

UVGlib::~UVGlib()
{
}

UVGlib *UVGlib::instance_ = nullptr;
void UVGlib::global_init()
{
    instance_ = new UVGlib();
    instance_->init();
}

void UVGlib::init()
{
    uv_prepare_init(uv_default_loop(), &this->prepare_handle_);
    this->prepare_handle_.data = this;
    uv_prepare_start(&this->prepare_handle_, UVGlib::prepare_cb);

    uv_check_init(uv_default_loop(), &this->check_handle_);
    this->check_handle_.data = this;
    uv_check_start(&this->check_handle_, UVGlib::check_cb);
}

void UVGlib::prepare_cb(uv_prepare_t *handle)
{
    gint timeout = 0;
    UVGlib *instance = (UVGlib *)handle->data;

    g_main_context_prepare(instance->main_context_, &instance->max_priority_);

    // Getting all sources from GLib main context
    while (true)
    {
        instance->nfds_ = g_main_context_query(instance->main_context_,
                                               instance->max_priority_,
                                               &timeout,
                                               instance->fds_,
                                               instance->allocated_nfds_);

        if (instance->allocated_nfds_ < instance->nfds_)
        {
            g_free(instance->fds_);
            g_free(instance->poll_handles_);

            instance->allocated_nfds_ = instance->nfds_;
            instance->fds_ = (GPollFD *)g_new(GPollFD, instance->allocated_nfds_);
            instance->poll_handles_ = (uv_poll_t *)g_new(uv_poll_t, instance->allocated_nfds_);
        }
        else
        {
            break;
        }
    }

    // Polling
    for (int i = 0; i < instance->nfds_; ++i)
    {
        GPollFD *poll_fd = instance->fds_ + i;
        uv_poll_t *handle = instance->poll_handles_ + i;
        int pevents = (poll_fd->events & G_IO_IN ? UV_READABLE : 0) | (poll_fd->events & G_IO_OUT ? UV_WRITABLE : 0);

        handle->data = poll_fd;
        poll_fd->revents = 0;

        uv_poll_init(uv_default_loop(), handle, poll_fd->fd);
        uv_poll_start(handle, pevents, UVGlib::poll_cb);
    }
}

void UVGlib::poll_cb(uv_poll_t *handle, int status, int events)
{
    GPollFD *pfd = (GPollFD *)handle->data;

    // KLOG_DEBUG("events: %d %d.", pfd->events, events);

    pfd->revents |= pfd->events & ((events & UV_READABLE ? G_IO_IN : 0) |
                                   (events & UV_WRITABLE ? G_IO_OUT : 0));
    uv_poll_stop(handle);
    uv_close((uv_handle_t *)handle, NULL);
}

void UVGlib::check_cb(uv_check_t *handle)
{
    UVGlib *instance = (UVGlib *)handle->data;
    // Release all polling events which aren't finished yet.
    for (int i = 0; i < instance->nfds_; ++i)
    {
        uv_poll_t *pt = instance->poll_handles_ + i;

        if (uv_is_active((uv_handle_t *)pt))
        {
            uv_poll_stop(pt);
            uv_close((uv_handle_t *)pt, NULL);
        }
    }

    g_main_context_check(instance->main_context_,
                         instance->max_priority_,
                         instance->fds_,
                         instance->nfds_);
    g_main_context_dispatch(instance->main_context_);
}

}  // namespace Kiran